local addonName, addonTable = ...

-- Adds to the addonTable:
--  OnHittableUpdated(avoidance)
--  ApplyLevelSettings()
--  ApplyDualWieldSettings()
-- Uses from the addonTable:
--  InitializeConfig()
--  UpdateTooltip()
--  DrawTooltip(enabled)
--  IsTooltipShown()
--  SetMobLevel(level)
--  SetDualWielding(state)
--  StartUpdating()
--  EndUpdating()

--------------------------
-- Constants -------------
--------------------------
local ICON_DW_DISABLED = "Interface\\Icons\\Ability_MeleeDamage"
local ICON_DW_ENABLED = "Interface\\Icons\\Ability_DualWield"

--------------------------
-- Variables -------------
--------------------------
local Addon = LibStub("AceAddon-3.0"):NewAddon(addonName)
local AceEvent = LibStub("AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale(addonName, false)
local db
local LDBObject = LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject(addonName, { type = "data source", label = L["LABEL"], text = "" })
local defaultLevel
local bossLevel

--------------------------
-- Upvalues --------------
--------------------------
local UnitIsEnemy, UnitLevel, UnitClassification, format = UnitIsEnemy, UnitLevel, UnitClassification, string.format
local UpdateTooltip, DrawTooltip, IsTooltipShown, SetMobLevel, SetDualWielding
local ApplyLevelSettings, ApplyDualWieldSettings, StartUpdating, EndUpdating, InitializeConfig

--------------------------
-- Functions -------------
--------------------------
function Addon:OnInitialize()
	UpdateTooltip, DrawTooltip, IsTooltipShown = addonTable.UpdateTooltip, addonTable.DrawTooltip, addonTable.IsTooltipShown
	SetMobLevel, SetDualWielding = addonTable.SetMobLevel, addonTable.SetDualWielding
	ApplyLevelSettings, ApplyDualWieldSettings = addonTable.ApplyLevelSettings, addonTable.ApplyDualWieldSettings
	StartUpdating, EndUpdating, InitializeConfig = addonTable.StartUpdating, addonTable.EndUpdating, addonTable.InitializeConfig
	table.wipe(addonTable)

	db = InitializeConfig()
	ApplyDualWieldSettings()
	self:SetEnabledState(db.char.enabled) -- Stop auto-enabling if addon is disabled
end

function Addon:OnEnable()
	ApplyLevelSettings()

	if IsTooltipShown() then
		DrawTooltip(true)
	end

	StartUpdating()

	db.char.enabled = true
end

function Addon:OnDisable()
	EndUpdating()
	LDBObject.text = ""
	AceEvent.UnregisterEvent(addonTable, "PLAYER_TARGET_CHANGED")

	if IsTooltipShown() then
		DrawTooltip(false)
	end

	db.char.enabled = false
end

function addonTable.ApplyDualWieldSettings()
	LDBObject.icon = db.char.dualwield and ICON_DW_ENABLED or ICON_DW_DISABLED
	SetDualWielding(db.char.dualwield)
end

function LDBObject.OnClick(frame, button)
	if Addon:IsEnabled() then
		if button == "LeftButton" then
			db.char.dualwield = not db.char.dualwield
			ApplyDualWieldSettings()
		else
			Addon:Disable()
		end
	else
		Addon:Enable()
	end
end

local function OnTargetChanged()
	if UnitIsEnemy("player", "target") then
		local level = UnitLevel("target")
		if level == -1 then
			if UnitClassification("target") == "worldboss" then
				level = bossLevel
			else
				level = UnitLevel("player") + 10
			end
		end
		SetMobLevel(level)
	else
		SetMobLevel(defaultLevel)
	end
end

function addonTable.ApplyLevelSettings()
	if not (db and db.char.enabled) then return end

	defaultLevel = db.profile.defaultlevel
	bossLevel = db.profile.bosslevel

	if db.profile.usetarget then
		AceEvent.RegisterEvent(addonTable, "PLAYER_TARGET_CHANGED", OnTargetChanged)
		OnTargetChanged() -- Force update
	else
		AceEvent.UnregisterEvent(addonTable, "PLAYER_TARGET_CHANGED")
		SetMobLevel(defaultLevel)
	end
end

function addonTable.OnHittableUpdated(avoidance)
	if IsTooltipShown() then
		UpdateTooltip()
	end
	LDBObject.text = format("%.2f%%", avoidance)
end
